@@ -106,7 +106,9 @@ public class GtkComponentPeer extends GtkGenericPeer
   native void gtkWidgetGetDimensions (int[] dim);
   native void gtkWidgetGetPreferredDimensions (int[] dim);
   native void gtkWindowGetLocationOnScreen (int[] point);
+  native void gtkWindowGetLocationOnScreenUnlocked (int[] point);
   native void gtkWidgetGetLocationOnScreen (int[] point);
+  native void gtkWidgetGetLocationOnScreenUnlocked (int[] point);
   native void gtkWidgetSetCursor (int type, GtkImage image, int x, int y);
   native void gtkWidgetSetCursorUnlocked (int type, GtkImage image,
                                           int x, int y);
@@ -252,9 +254,19 @@ public class GtkComponentPeer extends GtkGenericPeer
   { 
     int point[] = new int[2];
     if( this instanceof WindowPeer )
-      gtkWindowGetLocationOnScreen (point);
+      {
+        if (Thread.currentThread() == GtkMainThread.mainThread)
+          gtkWindowGetLocationOnScreenUnlocked (point);
+        else
+          gtkWindowGetLocationOnScreen (point);
+      }
     else
-      gtkWidgetGetLocationOnScreen (point);
+      {
+        if (Thread.currentThread() == GtkMainThread.mainThread)
+          gtkWidgetGetLocationOnScreenUnlocked (point);
+        else
+          gtkWidgetGetLocationOnScreen (point);
+      }
     return new Point (point[0], point[1]);
   }
 
@@ -616,11 +628,18 @@ public class GtkComponentPeer extends GtkGenericPeer
     setVisible (true);
   }
 
-  protected void postMouseEvent(int id, long when, int mods, int x, int y, 
+  protected void postMouseEvent(int id, long when, int mods, int x, int y,
                                 int clickCount, boolean popupTrigger) 
   {
-    q().postEvent(new MouseEvent(awtComponent, id, when, mods, x, y, 
-                                 clickCount, popupTrigger));
+    // It is important to do the getLocationOnScreen() here, instead
+    // of using the old MouseEvent constructors, because
+    // Component.getLocationOnScreen() locks on the AWT lock, which can
+    // trigger a deadlock. You don't want this.
+    Point locOnScreen = getLocationOnScreen();
+    q().postEvent(new MouseEvent(awtComponent, id, when, mods, x, y,
+                                 locOnScreen.x + x, locOnScreen.y + y,
+                                 clickCount, popupTrigger,
+                                 MouseEvent.NOBUTTON));
   }
 
   /**
@@ -899,4 +918,14 @@ public class GtkComponentPeer extends GtkGenericPeer
     // FIXME: implement
  
   }
+
+  public boolean requestFocus(Component lightweightChild, boolean temporary,
+                              boolean focusedWindowChangeAllowed,
+                              long time, sun.awt.CausedFocusEvent.Cause cause)
+  {
+    // TODO: Implement this properly and remove the other requestFocus()
+    // methods.
+    return true;
+  }
+
 }
