@@ -38,8 +38,6 @@ exception statement from your version. */
 
 package gnu.xml.dom.html2;
 
-import gnu.javax.swing.text.html.parser.support.Parser;
-
 import java.io.IOException;
 import java.io.Reader;
 
@@ -124,9 +122,11 @@ public class DomHTMLParser
     try
       {
         document = new DomHTMLDocument();
-
+        document.setCheckWellformedness(false);
+        document.setCheckingCharacters(false);
+        
         cursor = document;
-
+        
         parse(input);
 
         DomHTMLDocument h = document;
@@ -224,7 +224,6 @@ public class DomHTMLParser
               open.addFirst(close);
             close = close.getParentNode();
           }
-
         if (close == null)
           cursor = document;
         else
@@ -235,9 +234,8 @@ public class DomHTMLParser
         while (iter.hasNext())
           {
             Node item = (Node) iter.next();
-            Node copy = item.cloneNode(true);
-            cursor.appendChild(copy);
-            cursor = copy;
+            cursor.appendChild(item);
+            cursor = item;
           }
       }
   }
