@@ -1,5 +1,5 @@
 /* SAXParser.java -- 
-   Copyright (C) 2005, 2006  Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006, 2007  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -66,7 +66,6 @@ import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
 import org.xml.sax.ext.Attributes2;
 import org.xml.sax.ext.DeclHandler;
-import org.xml.sax.ext.EntityResolver2;
 import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.ext.Locator2;
 
@@ -187,6 +186,8 @@ public class SAXParser
       lexicalHandler = (LexicalHandler) value;
     else if ((GNU_FEATURES + "xml-base").equals(name))
       baseAware = Boolean.TRUE.equals(value);
+    else if ((GNU_FEATURES + "coalescing").equals(name))
+      coalescing = Boolean.TRUE.equals(value);
     else
       throw new SAXNotSupportedException(name);
   }
