@@ -379,7 +379,7 @@ public class ObjectInputStream extends InputStream
  	  readArrayElements(array, componentType);
  	  if(dump)
  	    for (int i = 0, len = Array.getLength(array); i < len; i++)
- 	      dumpElementln("  ELEMENT[" + i + "]=" + Array.get(array, i));
+ 	      dumpElementln("  ELEMENT[" + i + "]=", Array.get(array, i));
  	  ret_val = processResolution(null, array, handle, shared);
  	  break;
  	}
@@ -2083,6 +2083,24 @@ public class ObjectInputStream extends InputStream
     System.out.print (Thread.currentThread() + ": ");
   }
 
+  private void dumpElementln (String msg, Object obj)
+  {
+    try
+      {
+	System.out.print(msg);
+	if (java.lang.reflect.Proxy.isProxyClass(obj.getClass()))
+	  System.out.println(obj.getClass());
+	else
+	System.out.println(obj);
+      }
+    catch (Exception _)
+      {
+      }
+    for (int i = 0; i < depth; i++)
+      System.out.print (" ");
+    System.out.print (Thread.currentThread() + ": ");
+  }
+
   // used to keep a prioritized list of object validators
   private static final class ValidatorAndPriority implements Comparable
   {
