@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -50,7 +50,7 @@ import javax.xml.validation.Schema;
  * Factory for obtaining document builders.
  * Instances of this class are <em>not</em> guaranteed to be thread safe.
  *
- * @author (a href='mailto:dog@gnu.org'>Chris Burdess</a)
+ * @author <a href='mailto:dog@gnu.org'>Chris Burdess</a>
  */
 public abstract class DocumentBuilderFactory
 {
@@ -309,6 +309,7 @@ public abstract class DocumentBuilderFactory
   /**
    * Returns the schema.
    * @see #setSchema
+   * @since 1.5
    */
   public Schema getSchema()
   {
@@ -318,6 +319,7 @@ public abstract class DocumentBuilderFactory
   /**
    * Sets the schema.
    * @see #getSchema
+   * @since 1.5
    */
   public void setSchema(Schema schema)
   {
@@ -327,7 +329,7 @@ public abstract class DocumentBuilderFactory
   /**
    * Indicates whether parsers obtained from this factory will be XInclude
    * aware.
-   * @since 1.3
+   * @since 1.5
    */
   public boolean isXIncludeAware()
   {
@@ -336,11 +338,32 @@ public abstract class DocumentBuilderFactory
 
   /**
    * Sets whether parsers obtained from this factory will be XInclude aware.
-   * @since 1.3
+   * @since 1.5
    */
   public void setXIncludeAware(boolean state)
   {
     xIncludeAware = state;
   }
+
+  /**
+   * Sets the value of the specified feature.
+   * @param name the feature name (URI)
+   * @param value whether to enable the feature or not
+   * @exception ParserConfigurationException if the feature is not
+   * supported.
+   * @since 1.5
+   */
+  public abstract void setFeature(String name, boolean value)
+    throws ParserConfigurationException;
+
+  /**
+   * Returns the value of the specified feature.
+   * @param name the feature name (URI)
+   * @exception ParserConfigurationException if the feature is not
+   * supported.
+   * @since 1.5
+   */
+  public abstract boolean getFeature(String name)
+    throws ParserConfigurationException;
   
 }
