@@ -37,8 +37,13 @@ exception statement from your version. */
 
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GdkRobotPeer.h"
+
+#ifdef HAVE_XTEST
 #include <gdk/gdkx.h>
 #include <X11/extensions/XTest.h>
+#endif
+
+#ifdef HAVE_XTEST
 
 static int
 awt_button_mask_to_num (int buttons)
@@ -56,10 +61,15 @@ awt_button_mask_to_num (int buttons)
   return 0;
 }
 
+#endif
+
 JNIEXPORT jboolean JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_initXTest
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)))
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   int event_basep;
@@ -82,12 +92,22 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_initXTest
   gdk_threads_leave ();
 
   return result;
+
+#else
+
+  return JNI_FALSE;
+
+#endif
+
 }
 
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mouseMove
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)), jint x, jint y)
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   int result;
@@ -104,12 +124,23 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mouseMove
   XFlush (xdisplay);
 
   gdk_threads_leave ();
+
+#else
+
+  (void) x; // Unused.
+  (void) y; // Unused.
+
+#endif
+
 }
 
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mousePress
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)), jint buttons)
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   int result;
@@ -126,12 +157,22 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mousePress
   XFlush (xdisplay);
 
   gdk_threads_leave ();
+
+#else
+
+  (void) buttons; // Unused.
+
+#endif
+
 }
 
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mouseRelease
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)), jint buttons)
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   int result;
@@ -148,12 +189,22 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mouseRelease
   XFlush (xdisplay);
 
   gdk_threads_leave ();
+
+#else
+
+  (void) buttons; // Unused.
+
+#endif
+
 }
 
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mouseWheel
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)), jint wheelAmt)
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   int i = 0;
@@ -187,12 +238,22 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_mouseWheel
   XFlush (xdisplay);
 
   gdk_threads_leave ();
+
+#else
+
+  (void) wheelAmt; // Unused.
+
+#endif
+
 }
 
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_keyPress
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)), jint keycode)
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   GdkKeymapKey *keymap_keys = NULL;
@@ -232,12 +293,22 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_keyPress
   XFlush (xdisplay);
 
   gdk_threads_leave ();
+
+#else
+
+  (void) keycode; // Unused.
+
+#endif
+
 }
 
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GdkRobotPeer_keyRelease
   (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)), jint keycode)
 {
+
+#ifdef HAVE_XTEST
+
   GdkDisplay *display;
   Display *xdisplay;
   GdkKeymapKey *keymap_keys = NULL;
@@ -277,6 +348,13 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_keyRelease
   XFlush (xdisplay);
 
   gdk_threads_leave ();
+
+#else
+
+  (void) keycode; // Unused.
+
+#endif
+
 }
 
 JNIEXPORT jintArray JNICALL
@@ -284,6 +362,9 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_nativeGetRGBPixels
   (JNIEnv *env, jobject obj __attribute__((unused)), jint x, jint y,
    jint width, jint height)
 {
+
+#ifdef HAVE_XTEST
+
   jint stride_bytes, stride_pixels, n_channels, n_pixels;
   jintArray jpixels;  
   jint *java_pixels;
@@ -311,12 +392,8 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_nativeGetRGBPixels
   n_pixels = height * stride_pixels;
   gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
 
-  gdk_threads_leave ();
-
   jpixels = (*env)->NewIntArray (env, n_pixels);
 
-  gdk_threads_enter ();
-
   java_pixels = (*env)->GetIntArrayElements (env, jpixels, NULL);
 
   memcpy (java_pixels,
@@ -338,4 +415,16 @@ Java_gnu_java_awt_peer_gtk_GdkRobotPeer_nativeGetRGBPixels
   gdk_threads_leave ();
 
   return jpixels;
+
+#else
+
+  (void) env;
+  (void) x;
+  (void) y;
+  (void) width;
+  (void) height;
+  return NULL;
+
+#endif
+
 }
