@@ -40,11 +40,8 @@ exception statement from your version. */
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GtkWindowPeer.h"
 #include <gdk/gdkprivate.h>
-#include <gdk/gdkx.h>
-#include <X11/Xatom.h>
 #include <gdk/gdkkeysyms.h>
 
-#define AWT_WINDOW_OPENED 200
 #define AWT_WINDOW_CLOSING 201
 #define AWT_WINDOW_CLOSED 202
 #define AWT_WINDOW_ICONIFIED 203
@@ -55,6 +52,10 @@ exception statement from your version. */
 #define AWT_WINDOW_LOST_FOCUS 208
 #define AWT_WINDOW_STATE_CHANGED 209
 
+#define AWT_FRAME_NORMAL 0
+#define AWT_FRAME_ICONIFIED 1
+#define AWT_FRAME_MAXIMIZED_BOTH 6
+
 /* Virtual Keys */
 /* This list should be kept in the same order as the VK_ field
    declarations in KeyEvent.java. */
@@ -1036,17 +1037,10 @@ static void window_get_frame_extents (GtkWidget *window,
                                       int *top, int *left,
                                       int *bottom, int *right);
 
-static void request_frame_extents (GtkWidget *window);
-
-static Bool property_notify_predicate (Display *display,
-                                       XEvent  *xevent,
-                                       XPointer arg);
-
 static gboolean window_delete_cb (GtkWidget *widget, GdkEvent *event,
 			      jobject peer);
 static void window_destroy_cb (GtkWidget *widget, GdkEvent *event,
 			       jobject peer);
-static void window_show_cb (GtkWidget *widget, jobject peer);
 static void window_focus_state_change_cb (GtkWidget *widget,
                                           GParamSpec *pspec,
                                           jobject peer);
@@ -1148,12 +1142,6 @@ union extents_union
   unsigned long **extents;
 };
 
-union atom_list_union
-{
-  guchar **gu_extents;
-  Atom **atom_list;
-};
-
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GtkWindowPeer_create
   (JNIEnv *env, jobject obj, jint type, jboolean decorated, jobject parent)
@@ -1165,7 +1153,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_create
 
   gdk_threads_enter ();
   
-  NSA_SET_GLOBAL_REF (env, obj);
+  gtkpeer_set_global_ref (env, obj);
 
   window_widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   window = GTK_WINDOW (window_widget);
@@ -1173,7 +1161,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_create
   /* Keep this window in front of its parent, if it has one. */
   if (parent)
     {
-      window_parent = NSA_GET_PTR (env, parent);
+      window_parent = gtkpeer_get_widget (env, parent);
       gtk_window_set_transient_for (window, GTK_WINDOW(window_parent));
     }
 
@@ -1189,7 +1177,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_create
 
   gtk_widget_show (fixed);
 
-  NSA_SET_PTR (env, obj, window_widget);
+  gtkpeer_set_widget (env, obj, window_widget);
 
   gdk_threads_leave ();
 }
@@ -1203,7 +1191,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_gtkWindowSetTitle
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   c_title = (*env)->GetStringUTFChars (env, title, NULL);
 
@@ -1222,7 +1210,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_gtkWindowSetResizable
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
   gtk_window_set_resizable (GTK_WINDOW (ptr), resizable);
   g_object_set (G_OBJECT (ptr), "allow-shrink", resizable, NULL);
 
@@ -1237,7 +1225,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_gtkWindowSetModal
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   gtk_window_set_modal (GTK_WINDOW (ptr), modal);
 
@@ -1252,7 +1240,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_gtkWindowSetAlwaysOnTop
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   gtk_window_set_keep_above (GTK_WINDOW (ptr), alwaysOnTop);
 
@@ -1268,7 +1256,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_gtkWindowHasFocus
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   retval = gtk_window_has_toplevel_focus (GTK_WINDOW (ptr));
 
@@ -1296,7 +1284,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_setVisibleNativeUnlocked
 {
   void *ptr;
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   if (visible)
     gtk_widget_show (GTK_WIDGET (ptr));
@@ -1309,51 +1297,48 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_connectSignals
   (JNIEnv *env, jobject obj)
 {
   void *ptr;
-  jobject *gref;
+  jobject gref;
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
-  gref = NSA_GET_GLOBAL_REF (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
+  gref = (jobject) gtkpeer_get_global_ref (env, obj);
 
   g_signal_connect (G_OBJECT (ptr), "delete-event",
-		    G_CALLBACK (window_delete_cb), *gref);
+		    G_CALLBACK (window_delete_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "destroy-event",
-		    G_CALLBACK (window_destroy_cb), *gref);
-
-  g_signal_connect (G_OBJECT (ptr), "show",
-		    G_CALLBACK (window_show_cb), *gref);
+		    G_CALLBACK (window_destroy_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "notify::has-toplevel-focus",
-  		    G_CALLBACK (window_focus_state_change_cb), *gref);
+  		    G_CALLBACK (window_focus_state_change_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "focus-in-event",
-                    G_CALLBACK (window_focus_in_cb), *gref);
+                    G_CALLBACK (window_focus_in_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "focus-out-event",
-                    G_CALLBACK (window_focus_out_cb), *gref);
+                    G_CALLBACK (window_focus_out_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "window-state-event",
-		    G_CALLBACK (window_window_state_cb), *gref);
+		    G_CALLBACK (window_window_state_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "property-notify-event",
-		    G_CALLBACK (window_property_changed_cb), *gref);
+		    G_CALLBACK (window_property_changed_cb), gref);
 
   g_signal_connect_after (G_OBJECT (ptr), "realize",
-                          G_CALLBACK (realize_cb), *gref);
+                          G_CALLBACK (realize_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "key-press-event",
-                    G_CALLBACK (key_press_cb), *gref);
+                    G_CALLBACK (key_press_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "key-release-event",
-                    G_CALLBACK (key_release_cb), *gref);
+                    G_CALLBACK (key_release_cb), gref);
 
   g_signal_connect_after (G_OBJECT (ptr), "window-state-event",
-                          G_CALLBACK (window_window_state_cb), *gref);
+                          G_CALLBACK (window_window_state_cb), gref);
 
   g_signal_connect (G_OBJECT (ptr), "configure-event",
-                    G_CALLBACK (window_configure_cb), *gref);
+                    G_CALLBACK (window_configure_cb), gref);
 
   cp_gtk_component_connect_expose_signals (ptr, gref);
   cp_gtk_component_connect_mouse_signals (ptr, gref);
@@ -1373,7 +1358,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_realize (JNIEnv *env, jobject obj)
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   gtk_widget_realize (GTK_WIDGET (ptr));
 
@@ -1388,7 +1373,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_toBack (JNIEnv *env,
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
     
   gdk_window_lower (GTK_WIDGET (ptr)->window);
   gdk_flush ();
@@ -1404,7 +1389,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_toFront (JNIEnv *env,
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
     
   gdk_window_raise (GTK_WIDGET (ptr)->window);
   gdk_flush ();
@@ -1420,7 +1405,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_setSize
 
   gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   /* Avoid GTK runtime assertion failures. */
   width = (width < 1) ? 1 : width;
@@ -1449,7 +1434,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_nativeSetLocationUnlocked
 {
   void *ptr;
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   gtk_window_move (GTK_WINDOW(ptr), x, y);
 
@@ -1477,7 +1462,7 @@ Java_gnu_java_awt_peer_gtk_GtkWindowPeer_nativeSetBoundsUnlocked
   gint current_width;
   gint current_height;
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   /* Avoid GTK runtime assertion failures. */
   width = (width < 1) ? 1 : width;
@@ -1518,118 +1503,22 @@ static void
 window_get_frame_extents (GtkWidget *window,
                           int *top, int *left, int *bottom, int *right)
 {
-  unsigned long *extents = NULL;
-  union extents_union gu_ex;
+  GdkRectangle extents;
+  GdkWindow* gdkWindow;
 
-  /* Guess frame extents in case _NET_FRAME_EXTENTS is not
-     supported. */
-  if (!gtk_window_get_decorated (GTK_WINDOW (window)))
-    {
-      *top = 0;
-      *left = 0;
-      *bottom = 0;
-      *right = 0;
+  gint x, y, w, h;
 
-      return;
-    }
+  gdkWindow = window->window;
 
-  *top = 23;
-  *left = 6;
-  *bottom = 6;
-  *right = 6;
+  gdk_window_get_frame_extents(gdkWindow, &extents);
+  gdk_drawable_get_size(gdkWindow, &w, &h);
+  gdk_window_get_origin(gdkWindow, &x, &y);
 
-  /* Request that the window manager set window's
-     _NET_FRAME_EXTENTS property. */
-  request_frame_extents (window);
+  *left = extents.x - x;
+  *top = extents.y - y;
+  *right = extents.width - w - *left;
+  *bottom = extents.height - h - *top;
 
-  /* Attempt to retrieve window's frame extents. */
-  gu_ex.extents = &extents;
-  if (gdk_property_get (window->window,
-                        gdk_atom_intern ("_NET_FRAME_EXTENTS", FALSE),
-                        gdk_atom_intern ("CARDINAL", FALSE),
-                        0,
-                        sizeof (unsigned long) * 4,
-                        FALSE,
-                        NULL,
-                        NULL,
-                        NULL,
-                        gu_ex.gu_extents))
-    {
-      *left = extents [0];
-      *right = extents [1];
-      *top = extents [2];
-      *bottom = extents [3];
-    }
-}
-
-static Atom extents_atom = 0;
-
-/* Requests that the window manager set window's
-   _NET_FRAME_EXTENTS property. */
-static void
-request_frame_extents (GtkWidget *window)
-{
-  const char *request_str = "_NET_REQUEST_FRAME_EXTENTS";
-  GdkAtom request_extents = gdk_atom_intern (request_str, FALSE);
-
-  /* Check if the current window manager supports
-     _NET_REQUEST_FRAME_EXTENTS. */
-  if (gdk_net_wm_supports (request_extents))
-    {
-      GdkDisplay *display = gtk_widget_get_display (window);
-      Display *xdisplay = GDK_DISPLAY_XDISPLAY (display);
-
-      GdkWindow *root_window = gdk_get_default_root_window ();
-      Window xroot_window = GDK_WINDOW_XID (root_window);
-
-      Atom extents_request_atom =
-	gdk_x11_get_xatom_by_name_for_display (display, request_str);
-
-      XEvent xevent;
-      XEvent notify_xevent;
-
-      unsigned long window_id = GDK_WINDOW_XID (GDK_DRAWABLE(window->window));
-
-      if (!extents_atom)
-	{
-	  const char *extents_str = "_NET_FRAME_EXTENTS";
-	  extents_atom =
-	    gdk_x11_get_xatom_by_name_for_display (display, extents_str);
-	}
-
-      xevent.xclient.type = ClientMessage;
-      xevent.xclient.message_type = extents_request_atom;
-      xevent.xclient.display = xdisplay;
-      xevent.xclient.window = window_id;
-      xevent.xclient.format = 32;
-      xevent.xclient.data.l[0] = 0;
-      xevent.xclient.data.l[1] = 0;
-      xevent.xclient.data.l[2] = 0;
-      xevent.xclient.data.l[3] = 0;
-      xevent.xclient.data.l[4] = 0;
-
-      XSendEvent (xdisplay, xroot_window, False,
-		  (SubstructureRedirectMask | SubstructureNotifyMask),
-                  &xevent);
-
-      XIfEvent(xdisplay, &notify_xevent,
-	       property_notify_predicate, (XPointer) &window_id);
-    }
-}
-
-static Bool
-property_notify_predicate (Display *xdisplay __attribute__((unused)),
-                           XEvent  *event,
-                           XPointer window_id)
-{
-  unsigned long *window = (unsigned long *) window_id;
-
-  if (event->xany.type == PropertyNotify
-      && event->xany.window == *window
-      && event->xproperty.atom == extents_atom)
-    return True;
-  else
-    return False;
 }
 
 static gboolean
@@ -1660,16 +1549,6 @@ window_destroy_cb (GtkWidget *widget __attribute__((unused)),
 			      (jobject) NULL, (jint) 0);
 }
 
-static void
-window_show_cb (GtkWidget *widget __attribute__((unused)),
-		jobject peer)
-{
-  (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer,
-			      postWindowEventID,
-			      (jint) AWT_WINDOW_OPENED,
-			      (jobject) NULL, (jint) 0);
-}
-
 static void
 window_focus_state_change_cb (GtkWidget *widget,
 			      GParamSpec *pspec __attribute__((unused)),
@@ -1718,41 +1597,24 @@ window_window_state_cb (GtkWidget *widget __attribute__((unused)),
 			GdkEvent *event,
 			jobject peer)
 {
-  jint new_state;
-
-  /* Handle WINDOW_ICONIFIED and WINDOW_DEICONIFIED events. */
-  if (event->window_state.changed_mask & GDK_WINDOW_STATE_ICONIFIED)
-    {
-      /* We've either been iconified or deiconified. */
-      if (event->window_state.new_window_state & GDK_WINDOW_STATE_ICONIFIED)
-	{
-	  /* We've been iconified. */
-	  (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer,
-				      postWindowEventID,
-				      (jint) AWT_WINDOW_ICONIFIED,
-				      (jobject) NULL, (jint) 0);
-	}
-      else
-	{
-	  /* We've been deiconified. */
-	  (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer,
-				      postWindowEventID,
-				      (jint) AWT_WINDOW_DEICONIFIED,
-				      (jobject) NULL, (jint) 0);
-	}
-    }
-
-  /* Post a WINDOW_STATE_CHANGED event, passing the new frame state to
-     GtkWindowPeer. */
-  new_state = AWT_FRAME_STATE_NORMAL;
-
-  if (event->window_state.new_window_state & GDK_WINDOW_STATE_ICONIFIED)
-    new_state |= AWT_FRAME_STATE_ICONIFIED;
+  jint new_java_state = 0;
+  /* Put together the new state and let the java side figure out what
+   * to post */
+  GdkWindowState new_state = event->window_state.new_window_state;
+  /* The window can be either iconfified, maximized, iconified + maximized
+   * or normal. */
+  if ((new_state & GDK_WINDOW_STATE_ICONIFIED) != 0)
+    new_java_state |= AWT_FRAME_ICONIFIED;
+  if ((new_state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
+    new_java_state |= AWT_FRAME_MAXIMIZED_BOTH;
+  if ((new_state & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_ICONIFIED))
+      == 0)
+    new_java_state = AWT_FRAME_NORMAL;
 
   (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer,
 			      postWindowEventID,
 			      (jint) AWT_WINDOW_STATE_CHANGED,
-			      (jobject) NULL, new_state);
+			      (jobject) NULL, new_java_state);
 
   return TRUE;
 }
